#define _GNU_SOURCE 1
#include <errno.h>
#include <fcntl.h>
#include <stdlib.h>
#include "xl.h"
#include "xl_errno.c"
#define ARRAY_SIZE(x) (sizeof(x) / sizeof(*(x)))
#ifndef O_DIRECT
#	define O_DIRECT 0
#endif
#ifndef O_DIRECTORY
#	define O_DIRECTORY 0
#endif
#ifndef O_NOATIME
#	define O_NOATIME 0
#endif
#ifndef O_NOFOLLOW
#	define O_NOFOLLOW 0
#endif
#ifndef O_CLOEXEC
#	define O_CLOEXEC 0
#endif

enum {
	CCGFS_O_READ      = 1 << 0,
	CCGFS_O_WRITE     = 1 << 1,
	CCGFS_O_RDWR      = (CCGFS_O_READ | CCGFS_O_WRITE),
	CCGFS_O_ACCMODE   = CCGFS_O_RDWR,
	CCGFS_O_CREAT     = 1 << 2,
	CCGFS_O_EXCL      = 1 << 3,
	CCGFS_O_NOCTTY    = 1 << 4,
	CCGFS_O_TRUNC     = 1 << 5,
	CCGFS_O_APPEND    = 1 << 6,
	CCGFS_O_NONBLOCK  = 1 << 7,
	CCGFS_O_SYNC      = 1 << 8,
	CCGFS_O_ASYNC     = 1 << 9,
	CCGFS_O_DIRECT    = 1 << 10,
	CCGFS_O_DIRECTORY = 1 << 11,
	CCGFS_O_NOFOLLOW  = 1 << 12,
	CCGFS_O_NOATIME   = 1 << 13,
	CCGFS_O_CLOEXEC   = 1 << 14,
};

/* x is always negative or zero */
int generic_errno(int x)
{
	if (x > 0)
		abort();
	if (x < -ARRAY_SIZE(arch_to_generic_table))
		return x;
	else
		return arch_to_generic_table[-x];
}

/* x is always negative or zero */
int arch_errno(int x)
{
	if (x > 0)
		abort();
	if (x < -ARRAY_SIZE(generic_to_arch_table))
		return x;
	else
		return generic_to_arch_table[-x];
}

unsigned int generic_openflags(unsigned int x)
{
	unsigned int fl = 0;
	switch (x & O_ACCMODE) {
		case O_RDONLY:
			fl = CCGFS_O_READ;
			break;
		case O_WRONLY:
			fl = CCGFS_O_WRITE;
			break;
		case O_RDWR:
			fl = CCGFS_O_RDWR;
			break;
	}
	if (x & O_CREAT)     fl |= CCGFS_O_CREAT;
	if (x & O_EXCL)      fl |= CCGFS_O_EXCL;
	if (x & O_NOCTTY)    fl |= CCGFS_O_NOCTTY;
	if (x & O_TRUNC)     fl |= CCGFS_O_TRUNC;
	if (x & O_APPEND)    fl |= CCGFS_O_APPEND;
	if (x & O_SYNC)      fl |= CCGFS_O_SYNC;
	if (x & O_ASYNC)     fl |= CCGFS_O_ASYNC;
	if (x & O_DIRECT)    fl |= CCGFS_O_DIRECT;
	if (x & O_DIRECTORY) fl |= CCGFS_O_DIRECTORY;
	if (x & O_NOATIME)   fl |= CCGFS_O_NOATIME;
	if (x & O_NOFOLLOW)  fl |= CCGFS_O_NOFOLLOW;
	if (x & O_NONBLOCK)  fl |= CCGFS_O_NONBLOCK;
	if (x & O_CLOEXEC)   fl |= CCGFS_O_CLOEXEC;
	/* No encoding of O_LARGEFILE, will always enable */
	return fl;
}

unsigned int arch_openflags(unsigned int x)
{
	unsigned int fl = 0;
	switch (x & CCGFS_O_ACCMODE) {
		case CCGFS_O_READ:
			fl = O_RDONLY;
			break;
		case CCGFS_O_WRITE:
			fl = O_WRONLY;
			break;
		case CCGFS_O_RDWR:
			fl = O_RDWR;
			break;
	}
	fl |= O_LARGEFILE;
	if (x & CCGFS_O_CREAT)     fl |= O_CREAT;
	if (x & CCGFS_O_EXCL)      fl |= O_EXCL;
	if (x & CCGFS_O_NOCTTY)    fl |= O_NOCTTY;
	if (x & CCGFS_O_TRUNC)     fl |= O_TRUNC;
	if (x & CCGFS_O_APPEND)    fl |= O_APPEND;
	if (x & CCGFS_O_NONBLOCK)  fl |= O_NONBLOCK;
	if (x & CCGFS_O_SYNC)      fl |= O_SYNC;
	if (x & CCGFS_O_ASYNC)     fl |= O_ASYNC;
	if (x & CCGFS_O_DIRECT)    fl |= O_DIRECT;
	if (x & CCGFS_O_DIRECTORY) fl |= O_DIRECTORY;
	if (x & CCGFS_O_NOFOLLOW)  fl |= O_NOFOLLOW;
	if (x & CCGFS_O_NOATIME)   fl |= O_NOATIME;
	if (x & CCGFS_O_CLOEXEC)   fl |= O_CLOEXEC;
	return fl;
}
